@@ -1,5 +1,5 @@
 #
-# Copyright (c), 2018-2020, SISSA (International School for Advanced Studies).
+# Copyright (c), 2018-2021, SISSA (International School for Advanced Studies).
 # All rights reserved.
 # This file is distributed under the terms of the MIT License.
 # See the file 'LICENSE' in the root directory of the present
@@ -7,16 +7,17 @@
 #
 # @author Davide Brunato <brunato@sissa.it>
 #
+import sys
 import math
 import decimal
-from ..datatypes import Duration, DayTimeDuration, YearMonthDuration, StringProxy, AnyURI
+from ..datatypes import Duration, DayTimeDuration, YearMonthDuration, \
+    StringProxy, AnyURI, Float10
 from ..namespaces import XML_ID, XML_LANG, get_prefixed_name
-from ..xpath_nodes import TextNode, is_xpath_node, is_document_node, \
+from ..xpath_nodes import XPathNode, TextNode, is_xpath_node, is_document_node, \
     is_element_node, is_comment_node, is_processing_instruction_node, node_name
 
 from .xpath1_parser import XPath1Parser
 
-register = XPath1Parser.register
 method = XPath1Parser.method
 function = XPath1Parser.function
 
@@ -44,7 +45,12 @@ def select(self, context=None):
             yield context.item
         else:
             arg = self.get_argument(context, cls=str)
-            if context.item.tag == ' '.join(arg.strip().split()):
+            if hasattr(context.item, 'target'):
+                target = context.item.target
+            else:
+                target = context.item.text.split()[0] if context.item.text else ''
+
+            if target == ' '.join(arg.strip().split()):
                 yield context.item
 
 
@@ -79,26 +85,26 @@ def select(self, context=None):
 
 ###
 # Node set functions
-@method(function('last', nargs=0))
+@method(function('last', nargs=0, sequence_types=('xs:integer',)))
 def evaluate(self, context=None):
     if context is None:
         raise self.missing_context()
     return context.size
 
 
-@method(function('position', nargs=0))
+@method(function('position', nargs=0, sequence_types=('xs:integer',)))
 def evaluate(self, context=None):
     if context is None:
         raise self.missing_context()
     return context.position
 
 
-@method(function('count', nargs=1))
+@method(function('count', nargs=1, sequence_types=('item()*', 'xs:integer')))
 def evaluate(self, context=None):
     return len([x for x in self[0].select(context)])
 
 
-@method(function('id', nargs=1))
+@method(function('id', nargs=1, sequence_types=('xs:string*', 'element()*')))
 def select(self, context=None):
     if context is None:
         raise self.missing_context()
@@ -112,9 +118,9 @@ def select(self, context=None):
             yield from filter(lambda e: e.get(XML_ID) == value, item.iter())
 
 
-@method(function('name', nargs=(0, 1)))
-@method(function('local-name', nargs=(0, 1)))
-@method(function('namespace-uri', nargs=(0, 1)))
+@method(function('name', nargs=(0, 1), sequence_types=('node()?', 'xs:string')))
+@method(function('local-name', nargs=(0, 1), sequence_types=('node()?', 'xs:string')))
+@method(function('namespace-uri', nargs=(0, 1), sequence_types=('node()?', 'xs:anyURI')))
 def evaluate(self, context=None):
     if context is None:
         raise self.missing_context()
@@ -132,17 +138,17 @@ def evaluate(self, context=None):
     symbol = self.symbol
     if symbol == 'name':
         return get_prefixed_name(name, self.parser.namespaces)
-    elif not name or name[0] != '{':
-        return name if symbol == 'local-name' else ''
     elif symbol == 'local-name':
-        return name.split('}')[1]
-    elif symbol == 'namespace-uri':
-        return name.split('}')[0][1:]
+        return name if not name or name[0] != '{' else name.split('}')[1]
+    elif self.parser.version == '1.0':
+        return '' if not name or name[0] != '{' else name.split('}')[0][1:]
+    else:
+        return AnyURI('') if not name or name[0] != '{' else AnyURI(name.split('}')[0][1:])
 
 
 ###
 # String functions
-@method(function('string', nargs=(0, 1)))
+@method(function('string', nargs=(0, 1), sequence_types=('item()?', 'xs:string')))
 def evaluate(self, context=None):
     if not self:
         if context is None:
@@ -151,20 +157,22 @@ def evaluate(self, context=None):
     return self.string_value(self.get_argument(context))
 
 
-@method(function('contains', nargs=2))
+@method(function('contains', nargs=2,
+                 sequence_types=('xs:string?', 'xs:string?', 'xs:boolean')))
 def evaluate(self, context=None):
     arg1 = self.get_argument(context, default='', cls=str)
     arg2 = self.get_argument(context, index=1, default='', cls=str)
     return arg2 in arg1
 
 
-@method(function('concat', nargs=(2, None)))
+@method(function('concat', nargs=(2, None),
+                 sequence_types=('xs:anyAtomicType?', 'xs:anyAtomicType?', 'xs:string')))
 def evaluate(self, context=None):
     return ''.join(self.string_value(self.get_argument(context, index=k))
                    for k in range(len(self)))
 
 
-@method(function('string-length', nargs=(0, 1)))
+@method(function('string-length', nargs=(0, 1), sequence_types=('xs:string?', 'xs:integer')))
 def evaluate(self, context=None):
     if self:
         return len(self.get_argument(context, default_to_context=True, default='', cls=str))
@@ -175,7 +183,7 @@ def evaluate(self, context=None):
         raise self.missing_context() from None
 
 
-@method(function('normalize-space', nargs=(0, 1)))
+@method(function('normalize-space', nargs=(0, 1), sequence_types=('xs:string?', 'xs:string')))
 def evaluate(self, context=None):
     if self.parser.version == '1.0' or not self:
         arg = self.string_value(self.get_argument(context, default_to_context=True, default=''))
@@ -184,14 +192,16 @@ def evaluate(self, context=None):
     return ' '.join(arg.strip().split())
 
 
-@method(function('starts-with', nargs=2))
+@method(function('starts-with', nargs=2,
+                 sequence_types=('xs:string?', 'xs:string?', 'xs:boolean')))
 def evaluate(self, context=None):
     arg1 = self.get_argument(context, default='', cls=str)
     arg2 = self.get_argument(context, index=1, default='', cls=str)
     return arg1.startswith(arg2)
 
 
-@method(function('translate', nargs=3))
+@method(function('translate', nargs=3,
+                 sequence_types=('xs:string?', 'xs:string', 'xs:string', 'xs:string')))
 def evaluate(self, context=None):
     arg = self.get_argument(context, default='', cls=str)
 
@@ -214,7 +224,8 @@ def evaluate(self, context=None):
         return arg.translate(str.maketrans(map_string, trans_string[:len(map_string)]))
 
 
-@method(function('substring', nargs=(2, 3)))
+@method(function('substring', nargs=(2, 3),
+                 sequence_types=('xs:string?', 'xs:double', 'xs:double', 'xs:string')))
 def evaluate(self, context=None):
     item = self.get_argument(context, default='', cls=str)
     start = self.get_argument(context, index=1)
@@ -227,7 +238,7 @@ def evaluate(self, context=None):
         start = int(round(start)) - 1
 
     if len(self) == 2:
-        return '' if item is None else item[max(start, 0):]
+        return item[max(start, 0):]
     else:
         length = self.get_argument(context, index=2)
         try:
@@ -236,22 +247,20 @@ def evaluate(self, context=None):
         except TypeError:
             raise self.wrong_type("the third argument must be xs:numeric") from None
 
-        if item is None:
-            return ''
-        elif math.isinf(length):
+        if math.isinf(length):
             return item[max(start, 0):]
         else:
             stop = start + int(round(length))
-            return '' if item is None else item[slice(max(start, 0), max(stop, 0))]
+            return item[slice(max(start, 0), max(stop, 0))]
 
 
-@method(function('substring-before', nargs=2))
-@method(function('substring-after', nargs=2))
+@method(function('substring-before', nargs=2,
+                 sequence_types=('xs:string?', 'xs:string?', 'xs:string')))
+@method(function('substring-after', nargs=2,
+                 sequence_types=('xs:string?', 'xs:string?', 'xs:string')))
 def evaluate(self, context=None):
     arg1 = self.get_argument(context, default='', cls=str)
     arg2 = self.get_argument(context, index=1, default='', cls=str)
-    if arg1 is None:
-        return ''
 
     index = arg1.find(arg2)
     if index < 0:
@@ -264,27 +273,27 @@ def evaluate(self, context=None):
 
 ###
 # Boolean functions
-@method(function('boolean', nargs=1))
+@method(function('boolean', nargs=1, sequence_types=('item()*', 'xs:boolean')))
 def evaluate(self, context=None):
     return self.boolean_value([x for x in self[0].select(context)])
 
 
-@method(function('not', nargs=1))
+@method(function('not', nargs=1, sequence_types=('item()*', 'xs:boolean')))
 def evaluate(self, context=None):
     return not self.boolean_value([x for x in self[0].select(context)])
 
 
-@method(function('true', nargs=0))
+@method(function('true', nargs=0, sequence_types=('xs:boolean',)))
 def evaluate(self, context=None):
     return True
 
 
-@method(function('false', nargs=0))
+@method(function('false', nargs=0, sequence_types=('xs:boolean',)))
 def evaluate(self, context=None):
     return False
 
 
-@method(function('lang', nargs=1))
+@method(function('lang', nargs=1, sequence_types=('xs:string?', 'xs:boolean')))
 def evaluate(self, context=None):
     if context is None:
         raise self.missing_context()
@@ -311,7 +320,7 @@ def evaluate(self, context=None):
 
 ###
 # Number functions
-@method(function('number', nargs=(0, 1)))
+@method(function('number', nargs=(0, 1), sequence_types=('xs:anyAtomicType?', 'xs:double')))
 def evaluate(self, context=None):
     arg = self.get_argument(context, default_to_context=True)
     try:
@@ -320,24 +329,32 @@ def evaluate(self, context=None):
         return float('nan')
 
 
-@method(function('sum', nargs=(1, 2)))
+@method(function('sum', nargs=(1, 2),
+                 sequence_types=('xs:anyAtomicType*', 'xs:anyAtomicType?', 'xs:anyAtomicType?')))
 def evaluate(self, context=None):
-    values = [x[-1] if isinstance(x, tuple) else x for x in self[0].select(context)]
+    values = [x.value if isinstance(x, XPathNode) else x for x in self[0].select(context)]
     if not values:
         zero = 0 if len(self) == 1 else self.get_argument(context, index=1)
         return [] if zero is None else zero
 
     if all(isinstance(x, (decimal.Decimal, int)) for x in values):
-        return sum(values)
+        return sum(values) if len(values) > 1 else values[0]
     elif all(isinstance(x, DayTimeDuration) for x in values) or \
             all(isinstance(x, YearMonthDuration) for x in values):
-        return sum(values[1:], start=values[0])
+        if sys.version_info >= (3, 8):
+            return sum(values[1:], start=values[0])
+        result = values[0]
+        for val in values[1:]:
+            result += val
+        return result
     elif any(isinstance(x, Duration) for x in values):
         raise self.error('FORG0006', 'invalid sum of duration values')
     elif any(isinstance(x, (StringProxy, AnyURI)) for x in values):
         raise self.error('FORG0006', 'cannot apply fn:sum() to string-based types')
     elif any(isinstance(x, float) and math.isnan(x) for x in values):
         return float('nan')
+    elif all(isinstance(x, Float10) for x in values):
+        return sum(values)
 
     try:
         return sum(self.number_value(x) for x in values)
@@ -347,8 +364,8 @@ def evaluate(self, context=None):
         raise self.error('FORG0006') from None
 
 
-@method(function('ceiling', nargs=1))
-@method(function('floor', nargs=1))
+@method(function('ceiling', nargs=1, sequence_types=('numeric?', 'numeric?')))
+@method(function('floor', nargs=1, sequence_types=('numeric?', 'numeric?')))
 def evaluate(self, context=None):
     arg = self.get_argument(context)
     if arg is None:
@@ -370,7 +387,7 @@ def evaluate(self, context=None):
         raise self.error('FORG0006', err) from None
 
 
-@method(function('round', nargs=1))
+@method(function('round', nargs=1, sequence_types=('numeric?', 'numeric?')))
 def evaluate(self, context=None):
     arg = self.get_argument(context)
     if arg is None:
@@ -390,6 +407,8 @@ def evaluate(self, context=None):
     except TypeError as err:
         raise self.error('FORG0006', err) from None
     except decimal.InvalidOperation:
+        if isinstance(arg, str):
+            raise self.error('XPTY0004') from None
         return round(arg)
     except decimal.DecimalException as err:
         raise self.error('FOCA0002', err) from None
