@@ -13,15 +13,17 @@ import decimal
 import operator
 from copy import copy
 
-from ..exceptions import MissingContextError, ElementPathKeyError, xpath_error
+from ..helpers import EQNAME_PATTERN, normalize_sequence_type
+from ..exceptions import MissingContextError, ElementPathKeyError, \
+    ElementPathValueError, xpath_error
 from ..datatypes import AnyAtomicType, AbstractDateTime, Duration, DayTimeDuration, \
     YearMonthDuration, NumericProxy, ArithmeticProxy, UntypedAtomic, QName, \
     xsd10_atomic_types, xsd11_atomic_types, ATOMIC_VALUES
 from ..xpath_context import XPathSchemaContext
 from ..tdop import Parser
 from ..namespaces import XML_NAMESPACE, XSD_NAMESPACE, XPATH_FUNCTIONS_NAMESPACE, \
-    XSD_ANY_SIMPLE_TYPE, XSD_ANY_ATOMIC_TYPE, XSD_UNTYPED_ATOMIC, get_namespace, \
-    get_expanded_name, split_expanded_name
+    XPATH_MATH_FUNCTIONS_NAMESPACE, XSD_ANY_SIMPLE_TYPE, XSD_ANY_ATOMIC_TYPE, \
+    XSD_UNTYPED_ATOMIC, get_namespace, get_expanded_name, split_expanded_name
 from ..schema_proxy import AbstractSchemaProxy
 from ..xpath_token import XPathToken, XPathFunction
 from ..xpath_nodes import XPathNode, TypedElement, AttributeNode, TypedAttribute, \
@@ -108,6 +110,13 @@ class XPath1Parser(Parser):
     function_namespace = XPATH_FUNCTIONS_NAMESPACE
     function_signatures = {}
 
+    # https://www.w3.org/TR/xpath-3/#id-reserved-fn-names
+    RESERVED_FUNCTION_NAMES = {
+        'array', 'attribute', 'comment', 'document-node', 'element', 'empty-sequence',
+        'function', 'if', 'item', 'map', 'namespace-node', 'node', 'processing-instruction',
+        'schema-attribute', 'schema-element', 'switch', 'text', 'typeswitch',
+    }
+
     def __init__(self, namespaces=None, strict=True, *args, **kwargs):
         super(XPath1Parser, self).__init__()
         self.namespaces = self.DEFAULT_NAMESPACES.copy()
@@ -166,11 +175,41 @@ class XPath1Parser(Parser):
         )
 
     @classmethod
-    def function(cls, symbol, nargs=None, label='function', bp=90):
+    def function(cls, symbol, nargs=None, sequence_types=(), label='function', bp=90):
         """
         Registers a token class for a symbol that represents an XPath function.
         """
-        return cls.register(symbol, nargs=nargs, label=label,
+        if 'function' not in label:
+            pass
+        elif symbol in cls.RESERVED_FUNCTION_NAMES:
+            raise ElementPathValueError(f'{symbol!r} is a reserved function name')
+        elif sequence_types:
+            # Register function signature(s)
+            if label == 'math function':
+                qname = QName(XPATH_MATH_FUNCTIONS_NAMESPACE, 'math:%s' % symbol)
+            else:
+                qname = QName(XPATH_FUNCTIONS_NAMESPACE, 'fn:%s' % symbol)
+
+            if nargs is None:
+                pass  # pragma: no cover
+            elif isinstance(nargs, int):
+                assert len(sequence_types) == nargs + 1
+                cls.function_signatures[(qname, nargs)] = 'function({}) as {}'.format(
+                    ', '.join(sequence_types[:-1]), sequence_types[-1]
+                )
+            elif nargs[1] is None:
+                assert len(sequence_types) == nargs[0] + 1
+                cls.function_signatures[(qname, nargs[0])] = 'function({}, ...) as {}'.format(
+                    ', '.join(sequence_types[:-1]), sequence_types[-1]
+                )
+            else:
+                assert len(sequence_types) == nargs[1] + 1
+                for arity in range(nargs[0], nargs[1] + 1):
+                    cls.function_signatures[(qname, arity)] = 'function({}) as {}'.format(
+                        ', '.join(sequence_types[:arity]), sequence_types[-1]
+                    )
+
+        return cls.register(symbol, nargs=nargs, sequence_types=sequence_types, label=label,
                             bases=(XPathFunction,), lbp=bp, rbp=bp)
 
     def parse(self, source):
@@ -230,20 +269,66 @@ class XPath1Parser(Parser):
     def is_sequence_type(self, value):
         """Checks is a string is a sequence type specification."""
         try:
-            value = value.strip()
-        except AttributeError:
+            value = normalize_sequence_type(value)
+        except TypeError:
             return False
 
         if not value:
             return False
-        elif value == 'empty-sequence()':
+        elif value == 'empty-sequence()' or value == 'none':
             return True
-        elif value[-1] in ('?', '+', '*'):
+        elif value[-1] in {'?', '+', '*'}:
             value = value[:-1]
 
-        if value in {'untypedAtomic', 'attribute()', 'element()', 'text()', 'document-node()',
-                     'comment()', 'processing-instruction()', 'item()', 'node()'}:
+        if value in {'untypedAtomic', 'attribute()', 'attribute(*)', 'element()',
+                     'element(*)', 'text()', 'document-node()', 'comment()',
+                     'processing-instruction()', 'item()', 'node()', 'numeric'}:
             return True
+
+        elif value.startswith('element(') and value.endswith(')'):
+            if ',' not in value:
+                return EQNAME_PATTERN.match(value[8:-1]) is not None
+
+            try:
+                arg1, arg2 = value[8:-1].split(', ')
+            except ValueError:
+                return False
+            else:
+                return (arg1 == '*' or EQNAME_PATTERN.match(arg1) is not None) \
+                    and EQNAME_PATTERN.match(arg2) is not None
+
+        elif value.startswith('document-node(') and value.endswith(')'):
+            if not value.startswith('document-node(element('):
+                return False
+            return self.is_sequence_type(value[14:-1])
+
+        elif value.startswith('function('):
+            if value == 'function(*)' and self.version >= '3.0':
+                return True
+
+            try:
+                value, return_type = value.rsplit(' as ', 1)
+            except ValueError:
+                return False
+            else:
+                if not self.is_sequence_type(return_type):
+                    return False
+                elif value == 'function()':
+                    return True
+
+                value = value[9:-1]
+                if value.endswith(', ...'):
+                    value = value[:-5]
+
+                if 'function(' not in value:
+                    return all(self.is_sequence_type(x) for x in value.split(', '))
+
+                # Cover only if function() spec is the last argument
+                k = value.index('function(')
+                if not self.is_sequence_type(value[k:]):
+                    return False
+                return all(self.is_sequence_type(x) for x in value[:k].split(', ') if x)
+
         elif QName.pattern.match(value) is None:
             return False
 
@@ -298,7 +383,7 @@ class XPath1Parser(Parser):
         """
         if sequence_type[-1] in {'?', '+', '*'}:
             return self.match_sequence_type(value, sequence_type[:-1], sequence_type[-1])
-        elif value is None or value == []:
+        elif value is None or isinstance(value, list) and value == []:
             return sequence_type == 'empty-sequence()' or occurrence in {'?', '*'}
         elif sequence_type == 'empty-sequence()':
             return False
@@ -311,6 +396,8 @@ class XPath1Parser(Parser):
                 return all(self.match_sequence_type(x, sequence_type) for x in value)
         elif sequence_type == 'item()':
             return is_xpath_node(value) or isinstance(value, (AnyAtomicType, list))
+        elif sequence_type == 'numeric':
+            return isinstance(value, NumericProxy)
 
         value_kind = node_kind(value)
         if value_kind is not None:
@@ -325,15 +412,10 @@ class XPath1Parser(Parser):
     def check_variables(self, values):
         """Checks the sequence types of the XPath dynamic context's variables."""
         for varname, value in values.items():
-            if isinstance(value, list):
-                if self.match_sequence_type(value, 'item()', occurrence='*'):
-                    continue
-            else:
-                if self.match_sequence_type(value, 'item()'):
-                    continue
-
-            message = "Unmatched sequence type for variable {!r}".format(varname)
-            raise xpath_error('XPDY0050', message)
+            if not self.match_sequence_type(
+                    value, 'item()', occurrence='*' if isinstance(value, list) else None):
+                message = "Unmatched sequence type for variable {!r}".format(varname)
+                raise xpath_error('XPDY0050', message)
 
 
 ##
@@ -375,8 +457,15 @@ def nud(self):
             pass
         elif self.namespace == XSD_NAMESPACE:
             raise self.error('XPST0017', 'unknown constructor function {!r}'.format(self.value))
-        else:
+        elif self.value not in self.parser.RESERVED_FUNCTION_NAMES:
             raise self.error('XPST0017', 'unknown function {!r}'.format(self.value))
+        elif self.value == 'typeswitch':
+            msg = 'improper use of XQuery reserved name {!r}'
+            raise self.error('XPST0003', msg.format(self.value))
+        else:
+            msg = 'improper use of XPath reserved name {!r}'
+            raise self.error('XPST0017', msg.format(self.value))
+
     elif self.parser.next_token.symbol == '::':
         raise self.missing_axis("axis '%s::' not found" % self.value)
     return self
@@ -460,6 +549,8 @@ def led(self, left):
         try:
             namespace = self.get_namespace(left.value)
         except ElementPathKeyError:
+            self.parser.advance()  # Assure there isn't a following incomplete comment
+            self[:] = left, self.parser.token
             msg = "prefix {!r} is not declared".format(left.value)
             raise self.error('XPST0081', msg) from None
         else:
@@ -469,6 +560,10 @@ def led(self, left):
 
     self[:] = left, self.parser.expression(90)
     self.value = '{}:{}'.format(self[0].value, self[1].value)
+
+    if self.parser.next_token.symbol == ':':
+        raise self.wrong_syntax()
+
     return self
 
 
@@ -546,6 +641,11 @@ def nud(self):
 
     self[:] = self.parser.symbol_table['(string)'](self.parser, namespace), \
         self.parser.expression(90)
+
+    if self[1].value is None:
+        self.value = None
+    else:
+        self.value = '{%s}%s' % (self[0].value, self[1].value)
     return self
 
 
@@ -564,20 +664,19 @@ def select(self, context=None):
     elif context is None:
         raise self.missing_context()
 
-    name = '{%s}%s' % (self[0].value, self[1].value)
     if isinstance(context, XPathSchemaContext):
-        yield from self.select_xsd_nodes(context, name)
+        yield from self.select_xsd_nodes(context, self.value)
 
     elif self.xsd_types is None:
         for item in context.iter_children_or_self():
-            if match_attribute_node(item, name):
+            if match_attribute_node(item, self.value):
                 yield item
-            elif match_element_node(item, name):
+            elif match_element_node(item, self.value):
                 yield item
     else:
         # XSD typed selection
         for item in context.iter_children_or_self():
-            if match_attribute_node(item, name) or match_element_node(item, name):
+            if match_attribute_node(item, self.value) or match_element_node(item, self.value):
                 if isinstance(item, (TypedAttribute, TypedElement)):
                     yield item
                 else:
