@@ -19,6 +19,7 @@ from copy import copy
 from decimal import Decimal, DivisionByZero
 from urllib.parse import urlparse
 
+from ..helpers import normalize_sequence_type
 from ..exceptions import ElementPathError, ElementPathTypeError, \
     ElementPathValueError, MissingContextError, xpath_error
 from ..namespaces import XSD_NAMESPACE, XML_NAMESPACE, XLINK_NAMESPACE, \
@@ -215,7 +216,9 @@ class XPath2Parser(XPath1Parser):
         if not variable_types:
             self.variable_types = {}
         elif all(self.is_sequence_type(v) for v in variable_types.values()):
-            self.variable_types = variable_types.copy()
+            self.variable_types = {
+                k: normalize_sequence_type(v) for k, v in variable_types.items()
+            }
         else:
             raise ElementPathValueError('invalid sequence type for in-scope variable types')
 
@@ -294,20 +297,7 @@ class XPath2Parser(XPath1Parser):
         return self.next_token
 
     @classmethod
-    def signature(cls, value, *symbols, prefix=None):
-        arity = 0 if value.startswith('function()') else value.count(',') + 1
-        if not prefix:
-            for symbol in symbols:
-                qname = QName(XPATH_FUNCTIONS_NAMESPACE, 'fn:%s' % symbol)
-                cls.function_signatures[(qname, arity)] = value
-        else:
-            namespace = cls.DEFAULT_NAMESPACES[prefix]
-            for symbol in symbols:
-                qname = QName(namespace, '%s:%s' % (prefix, symbol))
-                cls.function_signatures[(qname, arity)] = value
-
-    @classmethod
-    def constructor(cls, symbol, bp=0, label='constructor function'):
+    def constructor(cls, symbol, bp=0, nargs=1, sequence_types=(), label='constructor function'):
         """Creates a constructor token class."""
         def nud_(self):
             try:
@@ -338,8 +328,13 @@ class XPath2Parser(XPath1Parser):
         def cast_(value):
             raise NotImplementedError
 
-        token_class = cls.register(symbol, nargs=1, label=label, bases=(XPathFunction,),
-                                   lbp=bp, rbp=bp, nud=nud_, evaluate=evaluate_, cast=cast_)
+        if not sequence_types:
+            assert nargs == 1
+            sequence_types = ('xs:anyAtomicType?', 'xs:%s?' % symbol)
+
+        token_class = cls.register(symbol, nargs=nargs, sequence_types=sequence_types,
+                                   label=label, bases=(XPathFunction,), lbp=bp, rbp=bp,
+                                   nud=nud_, evaluate=evaluate_, cast=cast_)
 
         def bind(func):
             assert func.__name__ == 'cast', \
@@ -427,15 +422,9 @@ class XPath2Parser(XPath1Parser):
             except KeyError:
                 sequence_type = 'item()*' if isinstance(value, list) else 'item()'
 
-            if sequence_type[-1] in ('?', '+', '*'):
-                if self.match_sequence_type(value, sequence_type[:-1], sequence_type[-1]):
-                    continue
-            else:
-                if self.match_sequence_type(value, sequence_type):
-                    continue
-
-            message = "Unmatched sequence type for variable {!r}".format(varname)
-            raise xpath_error('XPDY0050', message)
+            if not self.match_sequence_type(value, sequence_type):
+                message = "Unmatched sequence type for variable {!r}".format(varname)
+                raise xpath_error('XPDY0050', message)
 
 
 ##
